/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * irsndmain.c - demo main module to test irsnd encoder
 *
 * Copyright (c) 2010-2012 Frank Meyer - frank(at)fli4l.de
 *
 * ATMEGA88 @ 8 MHz
 *
 * Fuses: lfuse: 0xE2 hfuse: 0xDC efuse: 0xF9
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
#include "irsnd.h"
#include <avr/eeprom.h>

//#define ATMEL_AVR                                                                 // ATMEL AVR

#ifndef F_CPU
#  error F_CPU unkown
#else
# warning F_CPU OK
#endif


#define  nothing  0
#define  rising   1
#define  falling  2
#define  changing 3

#ifndef TRUE
	#define TRUE                          1
	#define FALSE                         0
#endif


#define ps2 0x10
#define T250 5000

// Member
// Tastencodes
unsigned int  ADRESS = 0x00FF;
unsigned int  VOL_UP = 0x28;     // Lautstrke +       9D62 28
unsigned int  VOL_DN = 0xA8;     // Lautstrke -
unsigned int  TRK_UP = 0x50;     // FWD
unsigned int  TRK_DN = 0xD0;     // RWD
unsigned int  SOURCE = 0xC8;     // Src wechseln
unsigned int  ANSWER = 0x68;     // - 20dB

int count = 0;

volatile int int0Operation = 0;
volatile int isSending = 0;

volatile unsigned long Tim2 = 0;

volatile char tCounter = 0;
const char tstartCounter = 4;

char bLearningMode = FALSE;
volatile unsigned int adcValue = 0;                   // Sample Ergebnis
volatile unsigned int adcValueOld = 0;
volatile char equalSample =0;
#define maxEqualSample 1

#define maxsteps 5
volatile  unsigned int  messwert[maxsteps] ;
volatile  unsigned int  lowmesswert[maxsteps] ;
volatile  int highmesswert[maxsteps] ;

volatile int step = 0;

// EEPROM variables
unsigned int   EEMEM eemesswert[maxsteps] ;
unsigned int   EEMEM eelowmesswert[maxsteps] ;
unsigned int  EEMEM eehighmesswert[maxsteps] ;


void timer1_init (void)
{
#if defined (__AVR_ATtiny45__) || defined (__AVR_ATtiny85__)                // ATtiny45 / ATtiny85:
    OCR1C   =  (F_CPU / F_INTERRUPTS / 4) - 1;                              // compare value: 1/15000 of CPU frequency, presc = 4
    TCCR1   = (1 << CTC1) | (1 << CS11) | (1 << CS10);                      // switch CTC Mode on, set prescaler to 4
#else                                                                       // ATmegaXX:
    OCR1A   =  (F_CPU / F_INTERRUPTS) - 1;                                  // compare value: 1/15000 of CPU frequency
    TCCR1B  = (1 << WGM12) | (1 << CS10);                                   // switch CTC Mode on, set prescaler to 1
#endif

#ifdef TIMSK1
    TIMSK1  = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#else
    TIMSK   = 1 << OCIE1A;                                                  // OCIE1A: Interrupt by timer compare
#endif
}


void startMyTimer(unsigned long iTicks)
{
	Tim2 = iTicks;
}

void stopMyTimer(void)
{
	Tim2 = 0;
}



//////////////////////////////////////////////////////////////////////////
void initadcfreeloop (void)
{

	equalSample = maxEqualSample;
	char cSREG = SREG;
	cli();

	ADMUX = ((1<<REFS0));
	// REFS0=1 & REFS1=0 : interne Vergleichsspannung VCC
	// ADLR = 0          : Messergebnis Left Adjust
	// MUX0 - MUX3 = 0      : Multiplexeingang 0

	ADCSRA |= (1 << ADEN);                             // AD Wandler freigeben
	ADCSRA |= (1 << ADFR);                             // freilaufende Wandlung
	ADCSRA |= ((1 << ADPS0) | (1 << ADPS1) | (1 << ADPS2));     // Prescaler Division Factor 128


	ADCSRA |= (1 << ADIE);                             // enable Interrupt
	ADCSRA |= (1 << ADIF);                             // enable Interrupt

	//sei();
	ADCSRA |= (1 << ADSC);                             // enable ADC (Wandlung starten)
	SREG = cSREG;										/* restore SREG value (I-bit) */
}

void stopadcfreeloop(void)
{
	char cSREG = SREG;
	cli();
	ADCSRA &= ~(1 << ADSC);                            // wandlung stoppen
	ADCSRA &= ~(1 << ADIE);                            // disable Interrupt
	ADCSRA &= ~(1 << ADIF);                            // disable Interrupt

	SREG = cSREG;										/* restore SREG value (I-bit) */
}
//////////////////////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////////////////////

void enableExtInt0falling(void)
{
	int0Operation = falling;
	// int 0
	// low level enable
	//MCUCR &= ~((1 << ISC00) | (1 << ISC01));
	//logical change
	//MCUCR |= (1 << ISC00);
	// falling edge
	MCUCR |= (1 << ISC01);
	MCUCR &= ~(1 << ISC00);
	// rising edge
	//MCUCR |= ((1 << ISC00) | (1 << ISC01));
	// int enable
	GICR |= (1 << INT0);
}
void enableExtInt0rising(void)
{
	int0Operation = rising;
	
	// int 0
	// low level enable
	//MCUCR &= ~((1 << ISC00) | (1 << ISC01));
	//logical change
	//MCUCR |= (1 << ISC00);
	// falling edge
	//MCUCR |= (1 << ISC01);
	// rising edge
	MCUCR |= ((1 << ISC00) | (1 << ISC01));
	// int enable
	GICR |= (1 << INT0);
}

void disableExtInt0(void)
{
	int0Operation = nothing;
	GICR &= ~(1 << INT0);
}

void enableExtInt1rising(void)
{
	// int 1
	// low level enable
	//MCUCR &= ~((1 << ISC10) | (1 << ISC11));
	//logical change
	//MCUCR |= (1 << ISC10);
	// falling edge
	//MCUCR |= (1 << ISC11);  // = 1
	//MCUCR &= ~(1 << ISC10); // = 0

	// rising edge
	MCUCR |= ((1 << ISC10) | (1 << ISC11));
	// int enable
	GICR |=(1 << INT1);
}

void enableExtInt1falling(void)
{
	// int 1
	// low level enable
	//MCUCR &= ~((1 << ISC10) | (1 << ISC11));
	//logical change
	//MCUCR |= (1 << ISC10);
	// falling edge
	MCUCR |= (1 << ISC11);     // = 1
	MCUCR &= ~(1 << ISC10);    // = 0

	// rising edge
	//MCUCR |= ((1 << ISC10) | (1 << ISC11));
	// int enable
	GICR |=(1 << INT1);
}
void disableExtInt1(void)
{
	GICR &= ~(1 << INT1);
}

void sendstring(unsigned int cmd)
{
	IRMP_DATA irmp_data;
	PORTB |=(1<<PINB2);					// Sende Led on
		
	irmp_data.protocol = IRMP_NEC_PROTOCOL;
	irmp_data.address  = 0x9D62;
	irmp_data.command  = cmd;
	irmp_data.flags    = 0;

	irsnd_send_data (&irmp_data, TRUE);
	_delay_us(50000);
	
	PORTB &= ~(1<<PINB2);				// Sende Led off
	
}

void computeLimits(void)
{
	volatile unsigned int idiff = 0;
	volatile unsigned int ihigh = 0;
	volatile unsigned int ilow = 0xFFFF;

	// hchsten und niedrigsten Messwert ermitteln

	for (int i = 0 ; i< maxsteps; i++)
	{
		if (messwert[i] > 0)
		{
			if(messwert[i] > ihigh)
			{
				ihigh = messwert[i];
			}

			if (messwert[i] < ilow)
			{
				ilow = messwert[i];
			}
		}
	}

	idiff = (ihigh - ilow) / (2 * (maxsteps+1));

	for (int i = 0; i < maxsteps; i++)
	{
		if (messwert[i] > 0)
		{
			lowmesswert[i] = messwert[i] - idiff;
			highmesswert[i] = messwert[i] + idiff;
		}
	}
}

void writetoEEprom(void)
{
	eeprom_write_block(&lowmesswert, &eelowmesswert, sizeof(lowmesswert));
	eeprom_write_block(&messwert, &eemesswert, sizeof(messwert));
	eeprom_write_block(&highmesswert, &eehighmesswert, sizeof(highmesswert));
}
//////////////////////////////////////////////////////////////////////////

/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * timer 1 compare handler, called every 1/10000 sec
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */
ISR(TIMER1_COMPA_vect)
{
    (void) irsnd_ISR();                                                     // call irsnd ISR
    // call other timer interrupt routines here...
	
	volatile unsigned long f = F_CPU;
	
	if (Tim2 != 0)
	{
		if (--Tim2 == 0)
		{
			PORTB ^= (1 << PINB0);		// Toggle the LED
			if (tCounter > 0)
			{
				tCounter--;
			}
			sendstring( VOL_UP);
			startMyTimer(2500);
			if(++count == 2)
			{
				count = 0;
				unsigned int i = (ADRESS && 0xFF);
				volatile unsigned int x = 0;
				i = ADRESS;
				i++;
				i &= 0xFF;
				x = (~(i << 8)& 0xFF00);
				x |= i;
				ADRESS = x;
				if (i == 0x00FF)
				{
					PORTB ^= (1<< PINB1);
				}
				else
				{
					//PORTB &= ~(1<< PINB1);
				}
			}
		}
	}

}


//////////////////////////////////////////////////////////////////////////
// ISR ADC
//////////////////////////////////////////////////////////////////////////

ISR (ADC_vect)
{
	// ADC Sample Ergebnis auswerten

	unsigned  int iData = 0;
	int index = -1;

	//adcValue = ADCL;                       // erst low dann highbyte lesen
	//adcValue |= (ADCH << 8);
	
	adcValue = ADC;
	
	//lowmesswert[0] = 90;
	//highmesswert[0] = 110;	


	if (isSending == 0)
	{
		if ( (adcValue > 0)&&( --equalSample == 0) && (bLearningMode == FALSE))
		{
			equalSample = maxEqualSample;
			iData = 0;		
			for (int i = 0; i < maxsteps; i++)
			{
				if ((adcValue > lowmesswert[i]) && (lowmesswert[i] > 0) && (adcValue < highmesswert[i]) )
				{
					index =i;
					break;
				}
			}

			if (index >= 0)
			{
				// Daten senden
				stopadcfreeloop();
				stopMyTimer();
				//PORTB &= ~(1<<PINB2);

				switch (index)
				{
					case 0:
						iData = VOL_DN;
						break;
					case 1:
						iData = VOL_UP;
						break;
					case 2:
						iData = TRK_DN;
						break;
					case 3:
						iData = TRK_UP;
						break;
					case 4:
						iData = SOURCE;
						break;
					case 5:
						iData = ANSWER;
						break;
					default:
						//initadcfreeloop();
						PORTB &= ~(1<<PINB2);            // senden fertig
						break;
				}

				if (iData != 0)
				{
					PORTB |=(1<<PINB2);					// Sende Led on
					isSending = 1;
					sendstring (iData);
					isSending = 0;
					PORTB &= ~(1<<PINB2);				// Sende Led off
				}
				initadcfreeloop();
			}
			else
			{
				PORTB &= ~(1<<PINB2);               // Sende Led off
				equalSample = maxEqualSample;
			}
		}
	}
}

// Taster Interrupt
ISR(INT0_vect)
{
	// ISR externer Interrupt 0

	// PINB0 Timer
	// PinB1 Ergebnis n mal blinken
	// PinB2 Taster Visu

	if (int0Operation == falling)
	{
		PORTB &= ~(1 << PINB2);       // Taster LED off
		PORTB &= ~(1 << PINB1);       // Ergebnis LED off

		tCounter = tstartCounter;     // 4 Zyklen sind erforderlich bevor Taste ausgewertet wird (entprellen)
		disableExtInt0();
		enableExtInt0rising();

		stopMyTimer();

		startMyTimer(T250);
		PORTB |=(1<<PINB0);
	}
	else if (int0Operation == rising)
	{
		PORTB &= ~(1<<PINB2);         // Taster on
		stopMyTimer();
		if (tCounter == 0)            // enable Learning mode
		{
			PORTB |= (1 << PINB1);     //  LED on
			bLearningMode = TRUE;
			step =0;
			enableExtInt1falling();    // einloggen des Wertes mit Int 1
		}
		disableExtInt0();
		enableExtInt0falling();
	}
	else
	{
		// nur Test, hier sollten wir nie landen
		PORTB |= (1 << PINB0)|(1 << PINB2);
		_delay_ms(1000);
		PORTB &= ~(1 << PINB0)|(1 << PINB2);
	}
}



ISR(INT1_vect)
{
	if (bLearningMode == TRUE)
	{
		if (step <= maxsteps)
		{
			messwert[step] = adcValue;

			for (int i = 0; i <= step; i++)
			{
				PORTB |= (1<<PINB2);
				_delay_ms(300);
				PORTB &= ~(1<<PINB2);
				_delay_ms(200);
			}
			step++;
		}

		if (step > maxsteps)       // Programmierung fertig
		{
			step=0;
			bLearningMode=FALSE;
			disableExtInt1();
			PORTB |= (1<<PINB1);
			_delay_ms(500);
			PORTB &= ~(1<<PINB1);
			_delay_ms(300);
			PORTB |= (1<<PINB1);
			_delay_ms(500);
			PORTB &= ~(1<<PINB1);
			computeLimits();          // Grenzwerte berechnen
			writetoEEprom();        // und im EEProm ablegen
		}
	}
}



/*---------------------------------------------------------------------------------------------------------------------------------------------------
 * MAIN: main routine
 *---------------------------------------------------------------------------------------------------------------------------------------------------
 */

int main (void)
{
    IRMP_DATA irmp_data;
	
    irsnd_init();																// initialize irsnd
    timer1_init();																// initialize timer

	DDRB |= ((1<<PINB0)|(1<<PINB1)|(1<<PINB2)|(1<<PINB3)|(1<<PINB4) );          // Set Pin as output
   	PORTB |= (1<<PINB5) |(1<<PINB6) ;															
	
	//PORTD = ((1<<PIND2) | (1<<PIND3) | (1<<PIND4));								// Pull up
	DDRD  = 0x00;																// Input
   
	eeprom_read_block(&lowmesswert, &eelowmesswert, sizeof(lowmesswert));
	eeprom_read_block(&highmesswert, &eehighmesswert, sizeof(highmesswert));
   
	//initadcfreeloop();                                       // ADC
	//enableExtInt0falling();                                  // Taster Event
   
    sei ();                                                  // enable global interrupts
	PORTB |= (1 << PINB0);                                   // LED Run
	startMyTimer(30000);
	while(1)
	{
	   asm volatile ("NOP");
	   volatile char i = PIND;
	   if ((i & (1<<PIND2)) == 0)
	   {
		
		
		   
	   }
	}
}
